#ifndef CORE_COMPONENTS_H
#define CORE_COMPONENTS_H

#include "core/HashMap.h"
#include "core/List.h"

typedef size_t CoreEntity;

typedef struct {
    CoreHashMap entityToIndex;
    CoreList indexToEntity;
    CoreList components;
} CoreComponents;

typedef struct {
    CoreEntity entity;
    void* component;
} CoreComponentNode;

typedef struct {
    const CoreEntity* indexToEntity;
    const CoreEntity* indexToEntityEnd;
    void* component;
    void* componentEnd;
    size_t componentSize;
    CoreComponentNode node;
} CoreComponentIterator;

#define CORE_COMPONENTS(componentSize)                                         \
    ((CoreComponents){CORE_HASH_MAP(sizeof(CoreEntity), sizeof(size_t),        \
                                    coreHash, coreEqual),                      \
                      CORE_LIST(sizeof(CoreEntity)),                           \
                      CORE_LIST(componentSize)})
void coreDestroyComponents(CoreComponents* c);
void* coreComponentsGetOrAdd(CoreComponents* c, CoreEntity e);
void* coreComponentsSearch(CoreComponents* c, CoreEntity e);
bool coreComponentsRemove(CoreComponents* c, CoreEntity e);
#define CORE_COMPONENTS_ITERATOR(c)                                            \
    ((CoreComponentIterator){coreListBegin(&(c)->indexToEntity),               \
                             coreListEnd(&(c)->indexToEntity),                 \
                             coreListBegin(&(c)->components),                  \
                             coreListEnd(&(c)->components),                    \
                             (c)->components.dataSize,                         \
                             {0}})
#define coreComponentsHasNext(ci) (ci.indexToEntity != ci.indexToEntityEnd)
CoreComponentNode* coreComponentsNext(CoreComponentIterator* ci);
void* coreComponentsBegin(CoreComponents* c);
void* coreComponentsEnd(CoreComponents* c);

#endif
